import { registerPaymentMethod } from '@woocommerce/blocks-registry';
import { __ } from '@wordpress/i18n';
import { getPaymentMethodData } from '@woocommerce/settings';
import { decodeEntities } from '@wordpress/html-entities';
import { useEffect } from '@wordpress/element';

const settings = getPaymentMethodData( 'transaction_cloud', {} );
const label = decodeEntities( settings?.title || '' ) || 'TransactionCloud payment';
const iconUrl = settings?.iconurl;
const paymentMethodsUrl = settings?.paymenturl;

const Content = props => {
	const { eventRegistration, emitResponse } = props;
	const { onCheckoutSuccess } = eventRegistration;

	useEffect( () => {
		const unsubscribe = onCheckoutSuccess(async (eventProps) => {

			if (props.activePaymentMethod != 'transaction_cloud') {
				return;
			}

			if (typeof window.tc_params === 'undefined') {
				return false;
			}

			if (typeof window.tc === 'undefined') {
				return false;
			}

			const customProductId = eventProps.processingResponse.paymentDetails.customProductId;
			// const popup = window.tc_params.popup_mode;

			try {
				// const ret = await tc.buy(customProductId, { popup });
				await tc.buy(customProductId);

				// if (ret.type && ret.type == 'CLOSE') {
				// 	console.log('Popup closed!');
				// 	return false;
				// }

				// if (ret.type && ret.type == 'TRANSACTION') {
				// 	window.location = data.transaction.url;
				// 	return true;
				// }
			} catch (e) {
				console.log('catch');

				return false;
			}

			return true;
		}, 1);

		return unsubscribe;
	}, [ onCheckoutSuccess ]);

	const alt = 'payment methods';
	const icon = <img src={ paymentMethodsUrl } alt={ alt } name={ alt } />
	return <div>{ icon }</div>;
};

const Label = props => {
	const { PaymentMethodLabel } = props.components;
	const icon = <img src={ iconUrl } alt={ label } name={ label } />
	return <PaymentMethodLabel className='transaction-cloud-block-label' text={ label } icon={ icon } />;
};

const transactionCloudPaymentMethod = {
	name: 'transaction_cloud',
	label: <Label />,
	content: <Content />,
	edit: <Content />,
	canMakePayment: () => true,
	ariaLabel: label,
	supports: {
		features: settings?.supports ?? [],
	}
};

registerPaymentMethod( transactionCloudPaymentMethod );
