jQuery( function( $ ) {
    // we need either tc_params or tc to continue
    if (typeof tc_params === 'undefined') {
		return false;
	}

    if (typeof tc === 'undefined') {
        return false;
    }

    if (typeof wc_checkout_params === 'undefined') {
        return false;
    }

    var wc_checkout_transaction_cloud = {
        $checkout_form: $( 'form.checkout' ),

        init: function() {
            this.$checkout_form.on( 'checkout_place_order_transaction_cloud', this.checkoutPlaceOrderHanlder);
        },

        checkoutPlaceOrderHanlder: function(e, wc_checkout_form) {
            const $form = $(this);

            $form.addClass( 'processing' );

            wc_checkout_form.blockOnSubmit( $form );

            // Attach event to block reloading the page when the form has been submitted
            wc_checkout_form.attachUnloadEventsOnSubmit();

            $.ajaxSetup( {
                dataFilter: function( raw_response, dataType ) {
                    // We only want to work with JSON
                    if ( 'json' !== dataType ) {
                        return raw_response;
                    }

                    if ( wc_checkout_form.is_valid_json( raw_response ) ) {
                        return raw_response;
                    } else {
                        // Attempt to fix the malformed JSON
                        var maybe_valid_json = raw_response.match( /{"result.*}/ );

                        if ( null === maybe_valid_json ) {
                            console.log( 'Unable to fix malformed JSON' );
                        } else if ( wc_checkout_form.is_valid_json( maybe_valid_json[0] ) ) {
                            console.log( 'Fixed malformed JSON. Original:' );
                            console.log( raw_response );
                            raw_response = maybe_valid_json[0];
                        } else {
                            console.log( 'Unable to fix malformed JSON' );
                        }
                    }

                    return raw_response;
                }
            } );

            $.ajax({
                type:		'POST',
                url:		wc_checkout_params.checkout_url,
                data:		$form.serialize(),
                dataType:   'json',
                success:	function( result ) {
                    // Detach the unload handler that prevents a reload / redirect
                    wc_checkout_form.detachUnloadEventsOnSubmit();

                    try {
                        if ( 'success' === result.result && $form.triggerHandler( 'checkout_place_order_success', [ result, wc_checkout_form ] ) !== false ) {
                            tc.buy(result.customProductId, { popup: tc_params.popup_mode }).then(
                                (data) => {
                                    $form.unblock();

                                    if (data.type == 'TRANSACTION') {
                                        window.location = data.transaction.url;
                                    }
                                },
                                () => {
                                    wc_checkout_form.submit_error( 'Failed to start transaction' );
                                }
                            );
                        } else if ( 'failure' === result.result ) {
                            throw 'Result failure';
                        } else {
                            throw 'Invalid response';
                        }
                    } catch( err ) {
                        // Reload page
                        if ( true === result.reload ) {
                            window.location.reload();
                            return;
                        }

                        // Trigger update in case we need a fresh nonce
                        if ( true === result.refresh ) {
                            $( document.body ).trigger( 'update_checkout' );
                        }

                        // Add new errors
                        if ( result.messages ) {
                            wc_checkout_form.submit_error( result.messages );
                        } else {
                            wc_checkout_form.submit_error( '<div class="woocommerce-error">' + wc_checkout_params.i18n_checkout_error + '</div>' ); // eslint-disable-line max-len
                        }
                    }
                },
                error:	function( jqXHR, textStatus, errorThrown ) {
                    // Detach the unload handler that prevents a reload / redirect
                    wc_checkout_form.detachUnloadEventsOnSubmit();

                    wc_checkout_form.submit_error(
                        '<div class="woocommerce-error">' +
                        ( errorThrown || wc_checkout_params.i18n_checkout_error ) +
                        '</div>'
                    );
                }
            });

            return false;
        },
    };

    wc_checkout_transaction_cloud.init();
});