<?php

if (!defined('ABSPATH')) {
    exit;
}

class TransactionCloudLogger
{
    private static $instance = null;
    private $stream;

    function __construct()
    {
        $this->stream = fopen($this->getFileName(), 'a');
    }

    function __destruct()
    {
        fclose($this->stream);
    }

    private function getFileName()
    {
        return sprintf(
            '%slogs/transactioncloud-%s.log',
            TC_WOO_PLUGIN_DIR,
            date('Y-m-d')
        );
    }

    public function write($record)
    {
        fwrite($this->stream, $record);
    }

    public static function getInstance()
    {
        if (static::$instance) {
            return static::$instance;
        }

        return static::$instance = new static();
    }

    public static function debug($message, $context = [])
    {
        static::log('DEBUG', $message, $context);
    }

    public static function error($message, $context = [])
    {
        static::log('ERROR', $message, $context);
    }

    public static function info($message, $context = [])
    {
        static::log('INFO', $message, $context);
    }

    private static function log($level, $message, $context = [])
    {
        $extra = '';
        if ($context) {
            try {
                $extra = json_encode($context, JSON_THROW_ON_ERROR);
            } catch (\Exception $e) {
                //
            }
        }

        $record = sprintf(
            '[%s][%s]: %s %s',
            date('Y-m-d H:i:s'),
            $level,
            $message,
            $extra
        ) . PHP_EOL;

        static::getInstance()->write($record);
    }
}
